package csp_runtime;

/**
 * Correspond a un tableau " la CSP", c'est--dire avec des bornes inf et sup
 * 
 * @author Benjamin LE BOZEC
 *
 * @param <T>
 */
public class CspArray<T> implements Valeur<CspArray<?>> {

	private Borne [] bornes;
	
	private Class maClass;
	
	CspData<?> [] tab;
	
	public CspArray (T initialValue, Borne ... bornes) {
		this.bornes = bornes;
		maClass = initialValue.getClass();
		
		int totalSize = 1;
		for (int i=bornes.length-1;i>=0;i--)
			totalSize = totalSize * bornes[i].getSize();
			
		tab = new CspData<?>[totalSize];
		for (int i=0;i<totalSize;i++)
			tab[i] = CspData.newCspData(initialValue);
	}
	
	public void set (T value, int ... index) {
		for (int i=0;i<index.length;i++) 
			if (index[i]<bornes[i].borneInf || index[i]>bornes[i].borneSup)
				throw new IndexOutOfBoundsException ("Dpassement du tableau avec l'indice " + index[i] +". (borneInf : " + bornes[i].borneInf + ", borneSup : " + bornes[i].borneSup+")");
		
		int lineSize = 1;
		int indice = 0;
		for (int i=index.length-1;i>=0;i--) {
			indice = indice + lineSize * (index[i]-bornes[i].borneInf);
			lineSize = lineSize*bornes[i].getSize();
		}
		
		((CspData<T>)tab[indice]).set(value);
	}
	
	public T get (int ... index) {
		for (int i=0;i<index.length;i++) 
			if (index[i]<bornes[i].borneInf || index[i]>bornes[i].borneSup)
				throw new IndexOutOfBoundsException ("Dpassement du tableau avec l'indice " + index[i] +". (borneInf : " + bornes[i].borneInf + ", borneSup : " + bornes[i].borneSup+")");
		
		int lineSize = 1;
		int indice = 0;
		for (int i=index.length-1;i>=0;i--) {
			indice = indice + lineSize * (index[i]-bornes[i].borneInf);
			lineSize = lineSize*bornes[i].getSize();
		}
		
		return (T) tab[indice].get();
	}

	public CspData<T> getData (int ... index) {
		for (int i=0;i<index.length;i++) 
			if (index[i]<bornes[i].borneInf || index[i]>bornes[i].borneSup)
				throw new IndexOutOfBoundsException ("Dpassement du tableau avec l'indice " + index[i] +". (borneInf : " + bornes[i].borneInf + ", borneSup : " + bornes[i].borneSup+")");
		
		int lineSize = 1;
		int indice = 0;
		for (int i=index.length-1;i>=0;i--) {
			indice = indice + lineSize * (index[i]-bornes[i].borneInf);
			lineSize = lineSize*bornes[i].getSize();
		}
		
		return (CspData<T>)tab[indice];
	}
	
	public boolean matches(CspArray<?> t) {
		if (t.maClass!=maClass) return false;
		if (bornes.length != t.bornes.length) return false;
		for (int i=0;i<bornes.length;i++)
			if (bornes[i].borneInf!=t.bornes[i].borneInf ||
				bornes[i].borneSup!=t.bornes[i].borneSup) return false;
		return true;
	}

	public void copyFrom(CspArray<?> t) {
		if (!matches(t)) throw new ClassCastException ("les 2 tableaux ne se correspondent pas !");
		for (int i=0;i<tab.length;i++)
			((CspData<T>)tab[i]).copyFrom((CspData<T>)t.tab[i]);
	}

	public CspArray<T> copy() {
		CspArray<T> t = new CspArray<T> ((T)tab[0].get(), bornes);
		t.copyFrom(this);
		return t;
	}
	
	public String toString () {
		String str = "[";
		for (int i=0;i<tab.length;i++) {
			str += tab[i].toString ();
			if (i!=tab.length-1) str += ", ";
		}
		
		str += "]";
		return str;
	}

	public static <T> CspArray<T> newCspArray(T i, Borne ... bornes) {
		return new CspArray<T>(i, bornes);
	}
			
}
