/*
 * Cr le 02/05/05
 */
package csp_runtime;

/**
 * CspData permet d'englober toute variable en vue de son utilisation dans une 
 * communication.
 * 
 * @author Benjamin LE BOZEC
 */
public class CspData<T> implements Valeur<CspData<?>> {

	/**
	 * Valeur associe au CspData
	 */
	private T value;
	
	/**
	 * Class associe au CspData (correspond au type de la valeur, soit 'T')
	 */
	private Class myClass;
	
	/**
	 * Constructeur. Assigne au CspData la valeur 'value' et la classe de 'value'.
	 * Attention : les valeurs nulles sont rejetes
	 * 
	 * @param value
	 */
	private CspData (T value) {
		if (value==null) throw new NullPointerException();
		this.value = value;
		this.myClass = value.getClass();
	}
	
	/**
	 * Constructeur. Assigne au CspData la classe 'c' et a 'value' la valeur null
	 * @param c
	 */
	private CspData (Class c) {
		if (c==null) throw new NullPointerException();
		this.value = null;
		this.myClass = c;
	}
	
	/**
	 * @return Returns the value.
	 */
	public T get() {
		return (T) value;
	}

	/**
	 * @param value The value to set.
	 */
	public void set(T value) {
		this.value = value;
	}

	/**
	 * @param d le CspData d'o la valeur doit tre rcupre
	 */
	public void copyFrom(CspData<?> d) {
		set ((T)d.get());
	}
	
	/**
	 * newCspData cr et initialise un nouvel objet CspData
	 * 
	 * @param value valeur par dfault
	 * @return l'objet CspData nouvellement cr
	 */
	public static <T> CspData<T> newCspData (T value) {
		return new CspData<T>(value);
	}
	
	/**
	 * newCspData cr et initialise un nouvel objet CspData en
	 * initialisant sa valeur a 'null'
	 * @param c la classe avec laquelle configurer le CspData
	 * @return l'objet CspData nouvellement cr
	 */
	public static <T> CspData<T> newCspData (Class<T> c, T value) {
		CspData<T> d = new CspData<T>(c);
		d.set (value);
		return d;
	}
	
	/**
	 * matches teste si le CspData donn en entre est du mme type que 
	 * le cspData actuel
	 * 
	 * @param d CspData dont la correspondance doit tre teste 
	 * @return vrai si ca correspond, et puis faux sinon..
	 */
	public boolean matches (CspData<?> d) {
		return d.myClass==myClass;
	}
	
	/**
	 * copy cr une nouvelle instance de CspData possdant la mm valeur et la 
	 * mm classe que le CspData actuel
	 * 
	 * @return le nouveau CspData
	 */
	public CspData<T> copy () {
		CspData<T> d = new CspData<T> (myClass);
		d.copyFrom(this);
		return d;
	}
	
	public String toString () {
		return value.toString ();
	}

	
}
