/*
 * Cr le 02/05/05
 */
package csp_runtime;

/**
 * Expression correspond  la notion d'expression en CSP.
 * Elle est donc dfinit par un type ("none" pour le 'brut'),
 * ainsi qu'un ensemble de valeurs (correspond ici a des CspData)
 * 
 * @author Benjamin LE BOZEC
 */
public class Expression implements Valeur<Expression> {

	/**
	 * Type associ a l'expression
	 */
	private String type;
	
	/**
	 * Donnes associes  l'expression
	 */
	private Valeur [] data;

	/**
	 * Constructeur
	 * @param t le type de l'expression
	 * @param data les donnes de l'expression
	 */
	public Expression (String type, Valeur ... data) {
		this.type = type;
		this.data = data;
	}
	
	/**
	 * matches teste si l'expression 'expr' correspond a l'expression actuelle.
	 * 
	 * Deux expressions se 'correspondent' si :
	 * 1) elle possde le mm type
	 * 2) elle possde le mm nombre de donnes
	 * 3) les donnes se correspondent (ont le mme typage) 
	 * 
	 * @param expr
	 * @return vrai si les 2 expressions se correspondent, faux sinon
	 */
	public boolean matches (Expression expr1) {
		Expression expr = (Expression) expr1;
		
		if (!this.type.equals(expr.type)) return false;
		
		if (this.data.length != expr.data.length) return false;
		
		for (int i=0;i<data.length;i++) {
			if (!data[i].matches(expr.data[i])) return false;
			/*if (data[i] instanceof CspData<?> && expr.data[i] instanceof CspData<?>) {
				CspData<?> d1 = (CspData<?>) data[i];
				CspData<?> d2 = (CspData<?>) expr.data[i];
				if (!d1.matches(d2)) return false;
			} else {
				Expression e1 = (Expression) data[i];
				Expression e2 = (Expression) expr.data[i];
				if (!e1.matches(e2)) return false;
			}*/
		}
		return true;
	}
	
	/**
	 * copy cr une copie de l'expression actuelle
	 * @return la nouvelle Expression cre
	 */
	public Expression copy () {
		Valeur [] tmp = new Valeur [data.length];
		
		for (int i=0;i<data.length;i++) {
			tmp[i] = (Valeur) data[i].copy();
			/*if (data[i] instanceof CspData<?>)
				tmp[i] = ((CspData<?>)data[i]).copy ();
			else 
				tmp[i] = ((Expression) data[i]).copy ();
				*/
		}
		
		return new Expression (type, tmp);
	}
	
	/**
	 * set permet d'attribuer a chaque CspData une nouvelle valeur.
	 * Afin de rsoudre les problmes de forme (x,y):=(y,x), on utilise
	 * un tampon intermdiaire.
	 * 
	 * Attention : les 2 expressions doivent correspondrent !!
	 * 
	 * @param expr
	 */
	public void copyFrom (Expression expr) {
		if (!matches(expr)) throw new ExpressionCastException ("Les deux expressions " + this + " et " + expr + " ne se correspondent pas");
		
		Expression tmp = expr.copy();

		for (int i=0;i<data.length;i++) {
			data[i].copyFrom (tmp.data[i]);
			/*if (data[i] instanceof CspData<?>) {
				CspData<?> d1 = (CspData<?>) data[i];
				CspData<?> d2 = (CspData<?>) tmp.data[i];
				d1.set (d2);
			} else { 
				Expression e1 = (Expression) data[i];
				Expression e2 = (Expression) tmp.data[i];
				e1.set (e2);
			}*/
		}
	}
	
	public String toString () {
		String descr = type + "(";
		for (int i=0;i<data.length;i++) {
			descr += data[i].toString() + ((i==data.length-1)?"":",");
		}
		return descr + ")";
	}

}
