/*
 * Cr le 3 mai 2005
 */
package csp_runtime;

/**
 * La classe Command correspond a une garde au sens CSP.
 * Elle possde donc une expression boolenne ainsi qu'une commande d'i/o,
 * sachant que l'une ou l'autre peuvent tre absente.
 * 
 * On considre qu'une commande d'i/o simple (c'est a dire qui ne fait pas
 * partie d'une garde d'une alternative) est une garde sans expression boolenne.
 * 
 * @author Benjamin LE BOZEC
 */
public class Garde {

	public final IOCommand ioCmd;
	public final BoolExpr bool;
	
	protected boolean choosen = false;
	protected int     num = -1;
	protected int []  range = {};
	
	private DebugObj debugObj;
	/**
	 * Constructeur. Attention, il faut au moins que l'un des 2 paramtres soit
	 * diffrent de null !!
	 * 
	 * @param garde
	 * @param ioCmd
	 */
	public Garde (BoolExpr garde,
					IOCommand ioCmd) {
		if (garde==null && ioCmd==null) throw new NullPointerException ();
		this.bool = garde;
		this.ioCmd = ioCmd;
	}
	
	/**
	 * Constructeur.
	 * 
	 * @param num le numro a attribuer  la commande (utile pour les alternatives)
	 * @param range 
	 * @param garde
	 * @param ioCmd
	 */
	public Garde (int num,
					int[] range, 
					BoolExpr garde,
					IOCommand ioCmd) {
		if (garde==null && ioCmd==null) throw new NullPointerException ();
		this.num = num;
		this.range = range;
		this.bool = garde;
		this.ioCmd = ioCmd;
		
		if (bool != null && ioCmd != null)
			debugObj = bool.getDebugObj().fusionne(ioCmd.debugObj);
		else {
			if (bool != null) 
				debugObj = bool.getDebugObj();
			else
				debugObj = ioCmd.debugObj;
		}	
	}

	/**
	 * @return VRAI si la commande a russi, faux sinon !
	 */
	public boolean getSuccess () {
		if (ioCmd != null) return ioCmd.getSuccess ();
		return true;
	}

	public String toString () {
		String descr = (bool!=null)?(bool.toString()+":"):"";
		descr += ioCmd.toString();
		return descr;
	}

	public DebugObj getDebugObj() {
		return debugObj;
	}
	
}
