/*
 * Cr le 3 mai 2005
 */
package csp_runtime;

import static csp_runtime.IOType.*;

/**
 * IOCommand reprsente une commande d'entre/sortie selon CSP.
 * 
 * @author Benjamin LE BOZEC
 */
public class IOCommand {

	public final CspThread  source;
	public final CspThread  dest;
	public final IOType     ioType;
	public final Expression expr;
	private boolean success;
	
	public DebugObj   debugObj;
	
	public IOCommand (CspThread source,
					  CspThread dest,
					  IOType    ioType,
					  Expression expr) {
		this (source, dest, ioType, expr, null);
	}

	public IOCommand (CspThread source,
			  CspThread dest,
			  IOType    ioType,
			  Expression expr,
			  DebugObj debugObj) {
		this.source = source;
		this.dest   = dest;
		this.ioType = ioType;
		this.expr   = expr;
		this.debugObj = debugObj;
	}

	/**
	 * dfinit si la commande a chou ou russi
	 * @param b 
	 */
	public void setSuccess (boolean b) {
		success = b;
	}
	
	/**
	 * @return success
	 */
	public boolean getSuccess() {
		return success;
	}

	/**
	 * @param ioCmd
	 * @return
	 */
	public boolean matches(IOCommand ioCmd) {
		if (ioCmd==null) return false;
		
		// 1) l'une des cmd est input, l'autre output
		if (ioType.getComplement() != ioCmd.ioType) return false;
		
		// 2) les 2 cmd se nomment mutuellement
		if (source != ioCmd.dest || dest != ioCmd.source) return false;
		
		// 3) Les 2 expressions des commandes correspondent
		if (!expr.matches(ioCmd.expr)) return false;
		
		return true;
	}

	/**
	 * @param command
	 */
	public void perform(IOCommand command) {
		if (command.ioType==IOType.INPUT) {
			command.expr.copyFrom(expr);
		} else {
			expr.copyFrom (command.expr);
		}
		setSuccess(true);
		command.setSuccess(true);
	}
	
	public String toString () {
		String descr = source + "." + dest;
		descr += (ioType==INPUT)?"?":"!";
		descr += expr;
		return descr;
	}
}
