/*
 * Cr le 2 mai 2005
 * 
 * Modifications :
 * - 09/05/05 : protection de la cration du select
 */
package csp_runtime;

import static csp_runtime.IOType.*;

import java.util.Vector;

/**
 * Select permet d'englober toutes les proprits relatives  une alternative,
 * cad les diffrentes gardes et cmd d'i/o correspond  un choix du select.
 *  
 * @author Benjamin LE BOZEC
 */
public class Select {

	/**
	 * thread source associ au select
	 */
	CspThread source;
	
	/**
	 * Liste des commandes associes au Select
	 */
	private Vector<Garde> commands = new Vector<Garde>();
	
	private int choosen = -1;
	private int[] range = null;	
	
	public DebugObj nextDebugObj = null;
	
	/**
	 * Constructeur.
	 * @param source le thread source a associer au select
	 */ 
	protected Select (CspThread source) {
		this.source = source;
	}
	
	/**
	 * addInput rfrence une commande d'entre dans le select.
	 * 
	 * @param garde la garde associe  la cmd
	 * @param dest le processus destination de la cmd
	 * @param expr l'expression cible de la communication
	 */
	public void addInput (int num,
						  int [] range,
						  BoolExpr garde, 
						  CspThread dest, 
						  Expression expr) {
		commands.add(new Garde(num, range, garde, new IOCommand (source, dest, INPUT, expr, nextDebugObj)));
		nextDebugObj = null;
	}

	/**
	 * addOutput rfrence une commande de sortie dans le select.
	 * 
	 * @param garde la garde associe  la cmd
	 * @param dest le processus destination de la cmd
	 * @param expr l'expression source de la communication
	 */
	public void addOutput (int num,
						   int [] range,
						   BoolExpr garde, 
						   CspThread dest, 
						   Expression expr) {
		commands.add(new Garde(num, range, garde, new IOCommand (source, dest, OUTPUT, expr, nextDebugObj)));
		nextDebugObj = null;
	}

	/**
	 * addSimpleGarde permet de rfrencer une simple expression boolene
	 * sans commande d'e/s.
	 * 
	 * @param garde la garde a referencer
	 */
	public void addSimpleGarde (int num, 
								int [] range,
								BoolExpr garde) {
		commands.add(new Garde(num, range, garde, null));
	}
	
	/**
	 * select () consistue l'valuation de l'alternative. Elle s'occupe donc
	 * d'envoyer au controleur les diffrentes commandes prcdemment rfrences
	 * par les mthodes addInput et addOutput. 
	 *  
	 * @return VRAI si l'une des commandes de l'alternative a t choisie,
	 *         FAUX si l'alternative a chou
	 */
	public boolean select () {
		Garde [] tab = new Garde [commands.size()];
		commands.toArray (tab);
		
		for (Garde cmd : commands) cmd.choosen = false;
		
		CspThread.controleur.manageCommands(source, tab);
		
		for (int i=0;i<tab.length;i++) 
			if (tab[i].choosen) {
				choosen = tab[i].num;
				range = tab[i].range;
				return true;
			}
				
		choosen = -1;
		return false;
	}
	
	/**
	 * @return l'indice de la commande choisie par l'alternative ou -1 si l'alternative
	 * a chhou. L'indice retourne dpend de l'ordre de referencement des commandes.
	 */
	public int getChoice () {
		return choosen;
	}
	
	public int [] getRange () {
		return range;
	}
	
}
